$(document).ready(function() {
    // Eigenes Kontextmenü des Users
    // Wenn length > 0 ist dann befindet sich der User auf einer Login-Seite
    // Darf vorher einfach nicht eingebunden werden wenn der Benutzer nicht eingeloggt ist.
    if ($("input[name='loginName']").length > 0) {
        console.log("User is on login page, no custom Context-Menu at this point.");
    }

    // Einstellungen
    settings = {
        animationType: "fade", // slide, fade, none
        animationSpeed: 500, // Animationsgeschwindigkeit (ms)
        hideContextMenuOnLeave: true, // true = Kontextmenü unsichtbar machen beim Fokus verlieren
        toleranceX: 10, // Toleranzgrenze horizontal fürs Positionieren
        toleranceY: 20, // Toleranzgrenze vertikal fürs Positionieren
    };

    switch (settings.animationType) {
        case "fade":
            show = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).fadeIn(speed);
            };
            hide = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).fadeOut(speed);
            };
            break;
        case "slide":
            show = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).slideDown(speed);
            };
            hide = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).slideUp(speed);
            };
            break;
        default:
            show = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).show(speed);
            };
            hide = function(element, speed) {
                if (speed === undefined)
                    speed = settings.animationSpeed;
                $(element).hide(speed);
            };
            break;
    }

    ctxMenu = {};
    ctxMenu.uniqueID = 1;
    ctxMenu.allItems = [];
    ctxMenu.actor = undefined;
    ctxMenu.html = $('<ul id="ctxMenu" class="ctxElement" style="display: none;"></ul>');
    setTimeout(function() { ctxMenu.html.appendTo("body"); }, 500);

    // Konstruktor für alle Kontext-Elemente, u.a. zum einfacheren setzen/holen von Eigenschaften
    ctxCtor = function(ret) {
        ret.ItemText = function(value) {
            if (value === undefined)
                return this.item.text();

            this.item.text(value);
        };
        ret.Attribute = function(attribute, value) {
            if (value === undefined)
                return this.item.attr(attribute);
            else
                this.item.attr(attribute, value);
        };
        ret.Toggle = function(enabled) {
            if (enabled === true || (enabled === undefined && this.hasClass("disabled")))
                this.removeClass("disabled");
            else
                this.addClass("disabled");
        };
        ret.IsDisabled = function() {
            return this.hasClass("disabled");
        };
        ret.ID = function() {
            return this.data("Item-ID");
        };
    };

    // Fügt ein neues Element dem Kontext-Menü hinzu
    // label  = Text
    // action = Script welches beim Klick ausgeführt wird, lässt sich auch im nachhinein mit anonymen Methoden zuweisen
    //          mittels item.Action = function(){};
    // sub    = kaskadieren in das Übergebene Untermenü, nichts übergeben um dem normalen Kontext-Menü (top-level) hinzuzufügen
    ctxMenu.addItem = function(label, action, sub) {
        var ret = $('<li class="ctxItem ctxElement"></li>');
        var retObj = $('<a href="#" class="ctxElement"></a>');
        retObj.appendTo(ret);
        ret.item = retObj;

        if (sub === undefined)
            ret.appendTo(ctxMenu.html);
        else
            ret.appendTo(sub.find("ul:first"));

        ctxCtor(ret);

        ret.Action = undefined;
        ret.Attribute("action", action);
        ret.ItemText(label);
        ctxMenu.assignID(ret);
        return ret;
    };

    // Fügt ein neues Untermenü hinzu
    // label        = Text
    // orientation  = top->Oben einfügen, bottom->Unten anhängen
    // sub          = kaskadieren in das Übergebene Untermenü, nichts übergeben um dem normalen Kontext-Menü (top-level) hinzuzufügen
    ctxMenu.addSub = function(label, orientation, sub) {
        var ret = $('<li class="ctxElement ctxSub"><div class="ctxElement ctxArrow">&gt;</div><ul class="ctxElement ctxSubList" style="display: none"></ul></li>');
        var item = $('<div class="ctxElement ctxSubLabel">' + label + ' </div>');
        item.appendTo(ret);

        ctxCtor(ret);

        var parentMenu = ctxMenu.html;
        if (sub !== undefined)
            parentMenu = $(sub).find("ul:first");

        if (orientation === 'bottom' || orientation === undefined || $(parentMenu).children().length === 0) {
            ret.appendTo(parentMenu);
        } else {
            ret.insertBefore($(parentMenu).children("li:first"));
        }

        return ret;
    };

    // Fügt ein neues Trennzeichen hinzu
    // sub = kaskadieren in das Übergebene Untermenü, nichts übergeben um dem normalen Kontext-Menü (top-level) hinzuzufügen
    ctxMenu.addSeparator = function(sub) {
        var ret = $('<li class="ctxElement ctxSeparator"><hr/></li>');

        if (sub === undefined)
            ret.appendTo(ctxMenu.html);
        else
            ret.appendTo(sub.find("ul:first"));

        return ret;
    };

    // Wird bei neuen Items automatisch ausgeführt
    ctxMenu.assignID = function(item) {
        if (item.ID() !== undefined)
            console.log("Item already has an ID: " + item.ID());

        item.data("Item-ID", ctxMenu.uniqueID);
        ctxMenu.uniqueID++;
        ctxMenu.allItems.push(item);
    };

    // Liefert ein Item mittels übergebener ID zurück (falls vorhanden)
    ctxMenu.getItem = function(id) {
        if (id !== undefined) {
            for (i = 0; i < ctxMenu.allItems.length; i++) {
                var item = ctxMenu.allItems[i];
                if (id === item.ID())
                    return item;
            }
        }

        return undefined;
    };

    // Hier weden die Aktionen ausgelöst
    $(document).on("click", "#ctxMenu li", function(event) {
        var jqThis = $(this);
        if (jqThis.hasClass("ctxSub"))
            return false;

        var sender = ctxMenu.getItem(jqThis.data("Item-ID"));
        if (sender === undefined || sender.Action === undefined)
            return true;

        event.preventDefault();
        if (sender.IsDisabled())
            return false;

        sender.Action(event, sender, ctxMenu.actor);

        hide(ctxMenu.html);
        return false;
    });

    $(document).on("mouseenter", "#ctxMenu li.ctxSub", function(event) {
        var sub = $(this);
        var subOffs = sub.offset();
        var ul = sub.children("ul.ctxSubList:first");

        if (ul.children("li.ctxItem").length === 0)
            return;

        show(ul, 0);

        // Positionierung auf dem Bildschirm falls nicht genügend Platz vorhanden ist,
        // z. B. wenn der Benutzer am Ende des Bildschirms klickt
        var height = ul.height();
        var width = ul.width();
        var y = subOffs.top;
        var x = subOffs.left + sub.width();
        if ((x + width) >= window.screen.availWidth)
            x = subOffs.left - ul.width();
        if ((y + height + 100 - window.scrollY) >= window.screen.availHeight)
            y = y - height;

        if (y < 0)
            y = 0;
        if (x < 0)
            x = 0;

        ul.offset({
            top: y,
            left: x
        });
    });

    // Untermenüsteuerung
    $(document).on("mouseleave", "#ctxMenu li.ctxSub", function(event) {
        var ul = $(this).children("ul.ctxSubList:first");
        hide(ul);
    });

    $(document).on("mouseenter", "#ctxMenu li.ctxSub", function(event) {
        var jqThis = $(this);
        var ul = jqThis.children("ul.ctxSubList:first");
        ul.stop();
        if (jqThis.find("li.ctxItem").length > 0)
            ul.show(0);
    });

    // Wird benötigt um das das Kontextmenü anzuzeigen
    // Von außern kann man auch mittels $("#ctxMenu").on("ctxMenuOpening", function(event, actor){}) darauf reagieren
    customContextMenuHandler = function(event) {
        event.preventDefault();

        ctxMenu.actor = $(event.target);

        var y = event.clientY - settings.toleranceY,
            x = event.clientX - settings.toleranceX;
        if ((x + ctxMenu.html.width()) >= window.screen.availWidth)
            x = x - ctxMenu.html.width() + (settings.toleranceX * 1.5);
        if ((y + ctxMenu.html.height() + 100) >= window.screen.availHeight)
            y = y - ctxMenu.html.height() + (settings.toleranceY * 1.5);

        ctxMenu.html.css({
            top: y,
            left: x
        });

        $("#ctxMenu").trigger("ctxMenuOpening", [ctxMenu.actor]);

        show(ctxMenu.html, 0);

        return false;
    };

    if (settings.hideContextMenuOnLeave) {
        $(document).on("mouseleave", "#ctxMenu", function() {
            hide(ctxMenu.html);
        });
        $(document).on("mouseenter", "#ctxMenu", function() {
            ctxMenu.html.stop();
            ctxMenu.html.show(0);
        });
    }

    // Klick woanders als im Kontext-Menü lässt das Menü verschwinden
    $(document).click(function(e) {
        if (!$(this).hasClass("ctxElement") && ctxMenu.html.is(":visible"))
            hide(ctxMenu.html, 0);
    });

    // Überschreibt das normale Kontext-Menü (wird mittels Strg ausgehebelt)
    $(document).on('contextmenu', function(event) {
        if (event.ctrlKey) {
            customContextMenuHandler(event);
        }
    });

    // Nun die Eigentlichen Elemente sammeln, kann/muss ggf. angepasst werden

    // Kontext-Menü
    var ignoredTitles = ["Hilfe", "Hilfe zu diesem Modul", "Abmelden"];

    // Persönliches Menu für Module
    // Manche Module sind dann 2 mal vorhanden, einmal angepinnt und einmal im Verlauf...
    // an anderer Stelle korrigieren wäre wohl sauberer
    var persMenu = $("a[data-context-menu='pers_menu']");
    AddMenu(persMenu, "Persönliches Menü");
    var histMenu = $("div#more ul li a");
    // var histMenu = $("a[data-context-menu='history']");
    AddMenu(histMenu, "Verlauf");
    ctxMenu.addSeparator();

    // Buttons - Als array of dict für leichteres erstellen und zuweisen
    // In der gewünschten Reihenfolge ins Array angeben
    var ctxDefaultButtons = [
        { name: "Neu", dataContextMenu: "create"},
        { name: "Speichern", dataContextMenu: "save"},
        { name: "Löschen", dataContextMenu: "delete"},
        { separator: true },
        { name: "Rückgängig", dataContextMenu: "undo"},
        { name: "Duplizieren", dataContextMenu: "duplicate"},
        { separator: true },
        { name: "Zurück", dataContextMenu: "ruecksprung"},
    /* Vorerst nicht verfügbar
        schließen nur über die Buttons und nicht übers Menü zulassen
        { name: "Modul schließen", dataContextMenu: "closeModule"},
        Hier sind außerdem die restlichen Standard-Buttons aber diese werden erstmal nicht ins Menüaufgenommen
        { name: "Aktualisieren", dataContextMenu: "refetchEO"},
        { name: "Seite einrichten", dataContextMenu: "enterConfig"},
        { name: "Satz merken", dataContextMenu: "bookmark"},
        { name: "anpinnen", dataContextMenu: "pin"},
    */
    ];

    ctxDefaultButtons.forEach(function(item) {
        AddItem(item);
    });

    // Administrative Buttons
    if ($("div.adminButtons").length === 1) {
        ctxMenu.addSeparator();
        var adminSub = ctxMenu.addSub("Administrative Aktionen");

        var ctxAdminButtons = [
            { name: "reloadScripts", dataContextMenu: "reloadScripts"},
            { name: "reloadChangedScripts", dataContextMenu: "reloadChangedScripts"},
            { name: "Session Debug an", dataContextMenu: "debugOn"},
            { name: "Log last transaction of my Session", dataContextMenu: "logSessionLastTransaction"},
            { name: "Log latest 500 lines", dataContextMenu: "log500"},
            { name: "Log complete", dataContextMenu: "log_komplett"},
            { name: "dbNames", dataContextMenu: "bu_dbnames"},
            { name: "Attributes", dataContextMenu: "bu_attributes"},
            { name: "Module", dataContextMenu: "bu_modul_def"},
            { name: "Scripts", dataContextMenu: "bu_modul_scripts"},
            { name: "Scriptfiles", dataContextMenu: "bu_modul_script_files"},
            { name: "Events", dataContextMenu: "bu_open_events"},
        ];

        ctxAdminButtons.forEach(function(item){
            AddItem(item, adminSub);
        });
    }

    // Kann verwendet werden um z. B. Elemente zu de-/aktivieren beim Öffnen des Menüs
    $("#ctxMenu").on("ctxMenuOpening", function(event, actor) {});

    function AddMenu(items, name) {
        /* Kommentierung entfernen um alphabetisch sortieren zu lassen (Home Button immer an oberster Stelle)
        items.sort(function(a, b) {
            var aText = $(a).attr("title") || $(a).text();
            var bText = $(b).attr("title") || $(b).text();

            if (aText === undefined)
                return -1;
            else if (bText === undefined)
                return 1;
            else if (aText === "")
                return -1;

            aText = aText.toUpperCase();
            bText = bText.toUpperCase();

            var result = aText > bText ? 1 : aText < bText ? -1 : 0;
            return result;
        });
        */

        var ctxSubModules = ctxMenu.addSub(name);

        if (items.length > 0) {
            for (i = 0; i < items.length; i++) {
                var moduleBtn = items[i];
                var modTitle = $(moduleBtn).attr("title") || $(moduleBtn).text();

                if (ignoredTitles.indexOf(modTitle) >= 0)
                    continue;

                var ctxModule = ctxMenu.addItem(modTitle, "ctxItemModule", ctxSubModules);
                ctxModule.Action = function(event, sender) { sender.ModuleButton.click(); };
                ctxModule.ModuleButton = moduleBtn;
                if (modTitle === "") {
                    ctxModule.css("font-family", "'Material Icons'");
                    ctxModule.css("font-size", "large");
                }
            }
        } else {
            ctxSubModules.Toggle(false);
        }
    }

    // Item muss entsprechend vordefiniert werden (siehe Zeile 309)
    function AddItem(item, sub){
        if(item.separator) {
            ctxMenu.addSeparator(sub);
        }
        else {
            var ctxItem = ctxMenu.addItem(item.name, "ctxItem" + item.dataContextMenu, sub);
            var ctxButton = $(document).find("a[data-context-menu='" + item.dataContextMenu + "']:first");
            if(ctxButton.length === 1) {
                ctxItem.Action = function() { ctxButton.get(0).click(); };
            }
            ctxItem.Toggle(ctxButton.length === 1);
        }
    }
});
